#'  auxiliary functions to compute the kernel test statistic.
#'
#' @details
#' The following auxiliary function is written to align the signs of the columns of \code{X} and \code{Y}
#' as the adjacency spectral embedding (under suitable assumptions)
#' are unique up to reflections of the columns.
#'
#' @param X \eqn{n x d} matrix
#' @param Y \eqn{n x d} matrix
#'
#' @return T a scalar or a vector of sign values
#' @author Minh Tang
#' @export

find.transform <- function(X,Y){
    u <- apply(X,2,median)
    v <- apply(Y,2,median)
    if(ncol(X) == 1){
        T <- sign(u/v)
    }
    else{
        T <- diag(sign(u/v))
    }
    T[is.na(T)] <- 1
    return(T)
}

