#'  auxiliary functions to compute the kernel test statistic.
#'
#' @details
#' Given an adjacency matrix \code{A} and embedding dimension \code{dmax}
#' and a vector \code{memb} of top-level memberships of the vertices of \code{A}
#' compute the adjacency spectral embedding for the subgraphs induced by the clusters in \code{memb}.
#'
#' @param A adjacency matrix
#' @param dmax embedding dimension
#' @param memb a vector of top-level membership of the vertices of \code{A}
#'
#' @return a list of matrices \code{Xhat[[1]]}, \code{Xhat[[2]]}, \code{Xhat[[3]]]}, ...
#' where \code{Xhat[[i]]} is the adjacency spectral embedding of the subgraph induced by \eqn{{v}} with \code{memb[v] = i}.
#' @author Minh Tang
#' @export

reembed <- function(A, dmax, memb, useSTFP=FALSE){
    X.list <- list()
    nc <- max(memb)
    for(i in 1:nc){
        idx.i <- which(memb == i)
        Ai <- A[idx.i,idx.i]
        if (useSTFP) {
            Xhat.i <- stfp(Ai,dmax)
        } else {
            Xhat.i <- embed_adjacency_matrix(graph.adjacency(Ai), dmax)$X
        }
#        Xhat.i <- stfp(Ai,dmax)
        ## eval <- sqrt(colSums(Xhat.i^2))
        ## dhat <- dimSelect(eval)
        X.list[[i]] <- Xhat.i
    }
    return(X.list)
}

#' @export
reembed.stfp <- function(A, dmax, memb){
    X.list <- list()
    nc <- max(memb)
    for(i in 1:nc){
        idx.i <- which(memb == i)
        Ai <- A[idx.i,idx.i]
#        Xhat.i <- embed_adjacency_matrix(graph.adjacency(Ai), dmax)$X
        Xhat.i <- stfp(Ai,dmax)
        ## eval <- sqrt(colSums(Xhat.i^2))
        ## dhat <- dimSelect(eval)
        X.list[[i]] <- Xhat.i
    }
    return(X.list)
}

