#' Make a random graph
#'
#' @details
#' Given a symmetric \eqn{n x n} matrix \eqn{P} whose entries are in \eqn{[0,1]}
#' generate a (random) symmetric adjacency matrix \eqn{A} whose entries are in \eqn{{0,1}}
#' such that \eqn{Pr[A_{ij} = 1] = P_{ij}} for \eqn{i < j}
#' The \eqn{{A_{ij}}_{i < j}} are independent random variables.
#'
#' @param P a symmetric \eqn{n x n} input matrix
#'
#' @return A a (random) symmetric adjacency matrix
#' @author Minh Tang
#' @export

rg.sample <- function(P){
  n <-  nrow(P)
  U <- matrix(0, nrow = n, ncol = n)
  U[col(U) > row(U)] <- runif(n*(n-1)/2)
  U <- (U + t(U))
  A <- (U < P) + 0 ;
  diag(A) <- 0
  return(A)
}

