library(hsbm)
library(igraph)
library(fpc)
library(popbio)

#source("http://www.cis.jhu.edu/~parky/HSBM/ccc_utils.r")

set.seed(1234)
R <- 8
(nR.vec <- 200+100*ceiling(5*runif(R)))
# Motif labels for the SBM
(motif.vec <- sample(1:3, R, replace = TRUE))
vlab <- rep(1:R, times=nR.vec)
mlab <- rep(motif.vec, times=nR.vec)

p <- 0.01

B.list <- list()
rho.list <- list()
for(r in 1:R){
    if(motif.vec[r] == 1){
        B.list[[r]] <- matrix(0.25,3,3);
        diag(B.list[[r]]) <- .4
        rho.list[[r]] <- c(0.25, 0.5, 0.25)

    } else if(motif.vec[r] == 2) {
        B.list[[r]] <- matrix(0.2,3,3);
        diag(B.list[[r]]) <- .25; B.list[[r]][2,2] <- .8
        rho.list[[r]] <- c(0.3, 0.4, 0.3)

    } else {
        B.list[[r]] <- matrix(0.25,3,3);
        diag(B.list[[r]]) <- 0.3; B.list[[r]][3,3] <- .7
        rho.list[[r]] <- c(0.4, 0.2, 0.4)
    }
}
rho.list
B.list

# subblock label
svec <- lapply(1:R, function(x) rho.list[[x]]*nR.vec[x])
slab <- unlist(sapply(1:R, function(x) rep(1:3,times=svec[[x]])+(x-1)*3))

# generate a graph
g <- sample_hierarchical_sbm(sum(nR.vec), nR.vec, rho.list, B.list, p)
mycol <- rainbow(max(mlab))[motif.vec]
plotmemb(g[],vlab,main=paste("A, R = ", max(vlab), ", m = 3"),drawborder=TRUE,lwd=.01,lcol=mycol,lwdb=2)

# Step 1: Embedding & initial clustering to get subgraphs Hhat
dmax <- 50
Xhat <- embed_adjacency_matrix(g,dmax,options=list(maxiter=10000))$X
eval <- sqrt(colSums(Xhat^2))
(dhat <- getElbows(eval,3,plot=F))
dhat <- dhat[1]

sXhat <- Xhat[,1:dhat] / sqrt(rowSums(Xhat[,1:dhat]^2))
Rmax <- 1.5*dhat
pamkout <- pamk(sXhat,krange=floor(Rmax/2):Rmax,usepam=FALSE)
pamkout$nc
membp <- pamkout$pamobj$cluster
(tablep <- table(membp))
mycol2 <- rainbow(max(membp))
plotmemb(g[],membp,main=paste("Skmeans, Rhat = ", max(membp)),drawborder=TRUE,lwd=.01,lcol=mycol2,lwdb=2)

# Step 2: Remebddding Hhat, compute Shat, and obtain motifs
sigma <- 0.5
X.list <- reembed(g[], 3, membp)
S <- computeS(X.list, sigma)
rownames(S) <- colnames(S) <- 1:pamkout$nc
my.image2(S[order(motif.vec), order(motif.vec)], text.cex = 0,
          round = 2, srt = 0, border = "gray70", label.cex = 1.5)

graphs.cluster <- pamk(S, diss = TRUE, krange = 2:(length(X.list)-1))
Yhat <- graphs.cluster$pamobject$clustering
(numc <- graphs.cluster$nc)

# Step 3: obtain motif parameters
Bhat.list <- list()
rhohat.list <- list()
for(i in 1:numc){
    idx.i <- which(Yhat == i)
    Xi <- NULL
    for(j in idx.i){
        if(is.null(Xi)){
            Xi <- X.list[[j]]
        } else {
            T <- find.transform(X.list[[j]], Xi)
            Xi <- rbind(Xi, X.list[[j]] %*% T)
        }
    }
    Xi.pamk <- pamk(Xi)
    Bi <- matrix(0, Xi.pamk$nc, Xi.pamk$nc)
    rhohat.list[[i]] <- as.vector(table(Xi.pamk$pamobj$cluster))/nrow(Xi)
    for(j1 in 1:Xi.pamk$nc){
        for(j2 in j1:Xi.pamk$nc){
            Bi[j1,j2] <- sum(Xi.pamk$pamobj$medoids[j1,]*Xi.pamk$pamobj$medoids[j2,])
            Bi[j2,j1] <- Bi[j1,j2]
        }
    }
    Bhat.list[[i]] <- Bi
}
Bhat.list
rhohat.list
